__all__ = [
    "button",
    "checkbox",
    "date",
    "dialog",
    "image",
    "img",
    "input",
    "number",
    "radio",
    "range",
    "select",
    "a",
    "abbr",
    "address",
    "area",
    "article",
    "aside",
    "audio",
    "b",
    "bdi",
    "bdo",
    "blockquote",
    "br",
    "canvas",
    "caption",
    "cite",
    "code",
    "col",
    "colgroup",
    "data",
    "datalist",
    "dd",
    "del_",
    "details",
    "dfn",
    "div",
    "dl",
    "dt",
    "em",
    "embed",
    "fieldset",
    "figcaption",
    "figure",
    "footer",
    "form",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "header",
    "hr",
    "i",
    "iframe",
    "ins",
    "kbd",
    "label",
    "legend",
    "li",
    "main",
    "map_",
    "mark",
    "menu",
    "meta",
    "meter",
    "nav",
    "noscript",
    "object_",
    "ol",
    "optgroup",
    "option",
    "output",
    "p",
    "param",
    "picture",
    "pre",
    "progress",
    "q",
    "rp",
    "rt",
    "ruby",
    "s",
    "samp",
    "script",
    "search",
    "section",
    "small",
    "source",
    "span",
    "strong",
    "style",
    "sub",
    "summary",
    "sup",
    "svg",
    "tbody",
    "td",
    "template",
    "tfoot",
    "th",
    "thead",
    "time",
    "title",
    "tr",
    "track",
    "u",
    "var",
    "wbr",
    "paragraph",
    "table",
    "textarea",
    "ul",
    "video",
]

from .button import Button as button
from .checkbox import Checkbox as checkbox
from .date import Date as date
from .dialog import Dialog as dialog
from .image import Image as image
from .image import Image as img
from .input import Input as input
from .number import Number as number
from .radio import Radio as radio
from .range import Range as range
from .select import Select as select
from .simple import (
    a,
    abbr,
    address,
    area,
    article,
    aside,
    audio,
    b,
    bdi,
    bdo,
    blockquote,
    br,
    canvas,
    caption,
    cite,
    code,
    col,
    colgroup,
    data,
    datalist,
    dd,
    del_,
    details,
    dfn,
    div,
    dl,
    dt,
    em,
    embed,
    fieldset,
    figcaption,
    figure,
    footer,
    form,
    h1,
    h2,
    h3,
    h4,
    h5,
    h6,
    header,
    hr,
    i,
    iframe,
    ins,
    kbd,
    label,
    legend,
    li,
    main,
    map_,
    mark,
    menu,
    meta,
    meter,
    nav,
    noscript,
    object_,
    ol,
    optgroup,
    option,
    output,
    p,
    param,
    picture,
    pre,
    progress,
    q,
    rp,
    rt,
    ruby,
    s,
    samp,
    script,
    search,
    section,
    small,
    source,
    span,
    strong,
    style,
    sub,
    summary,
    sup,
    svg,
    tbody,
    td,
    template,
    tfoot,
    th,
    thead,
    time,
    title,
    tr,
    track,
    u,
    var,
    wbr,
)
from .simple import (
    p as paragraph,
)
from .table import Table as table
from .textarea import Textarea as textarea
from .ul import Ul as ul
from .video import Video as video
