{% from 'debug/sse.html.j2' import build_sse as build_debug_sse -%}

<!doctype html>
<html lang="en">

<head>
  <meta charset="UTF-8" />
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  <link href="{{ model.favicon_url }}" rel="shortcut icon" />
  <title>500 - server error</title>
  <style>
      .title { color: #e74c3c;font-family: Arial, sans-serif; text-align: center; }
      .error-console {
        background-color: #1e1e1e;  
        color: #f8f8f2;            
        padding: 15px;
        border-radius: 5px; 
        font-size: 1rem;
        line-height: 1.5;
        overflow-x: auto;            
        white-space: pre-wrap;
        word-wrap: break-word;
        border: 1px solid #444;
        margin: 20px 0;
      }
  </style>
</head>

<body>
  {{ build_debug_sse(model.is_debug, model.prefix) }}
  <h1 class="title">500 - server error</h1>
  <pre class="error-console">{{ model.exception_format()|e }}</pre>
</body>
</html>